Attribute VB_Name = "modGDIP"
Option Explicit

' ----==== GDIPlus Const ====----
Private Const GdiPlusVersion As Long = 1&

' ----==== GDIPlus Types ====----
Private Type GDIPlusStartupInput
    GdiPlusVersion As Long
    DebugEventCallback As Long
    SuppressBackgroundThread As Long
    SuppressExternalCodecs As Long
End Type

Private Type GdiplusStartupOutput
    NotificationHook As Long
    NotificationUnhook As Long
End Type

Public Type RECTF
    Left As Single
    Top As Single
    Width As Single
    Height As Single
End Type

' ----==== GDIPlus Enums ====----
Public Enum Status
    OK = 0
    GenericError = 1
    InvalidParameter = 2
    OutOfMemory = 3
    ObjectBusy = 4
    InsufficientBuffer = 5
    NotImplemented = 6
    Win32Error = 7
    WrongState = 8
    Aborted = 9
    FileNotFound = 10
    ValueOverflow = 11
    AccessDenied = 12
    UnknownImageFormat = 13
    FontFamilyNotFound = 14
    FontStyleNotFound = 15
    NotTrueTypeFont = 16
    UnsupportedGdiplusVersion = 17
    GdiplusNotInitialized = 18
    PropertyNotFound = 19
    PropertyNotSupported = 20
    ProfileNotFound = 21
End Enum

Public Enum FillMode
    FillModeAlternate = &H0
    FillModeWinding = &H1
End Enum

Public Enum MatrixOrder
    MatrixOrderPrepend = &H0
    MatrixOrderAppend = &H1
End Enum

Public Enum Unit
    UnitWorld = &H0
    UnitDisplay = &H1
    UnitPixel = &H2
    UnitPoint = &H3
    UnitInch = &H4
    UnitDocument = &H5
    UnitMillimeter = &H6
End Enum

Public Enum PixelFormats
    PixelFormatUndefined = &H0&
    PixelFormatDontCare = PixelFormatUndefined
    PixelFormatMax = &HF&
    PixelFormat1_8 = &H100&
    PixelFormat4_8 = &H400&
    PixelFormat8_8 = &H800&
    PixelFormat16_8 = &H1000&
    PixelFormat24_8 = &H1800&
    PixelFormat32_8 = &H2000&
    PixelFormat48_8 = &H3000&
    PixelFormat64_8 = &H4000&
    PixelFormat16bppRGB555 = &H21005
    PixelFormat16bppRGB565 = &H21006
    PixelFormat16bppGrayScale = &H101004
    PixelFormat16bppARGB1555 = &H61007
    PixelFormat24bppRGB = &H21808
    PixelFormat32bppRGB = &H22009
    PixelFormat32bppARGB = &H26200A
    PixelFormat32bppPARGB = &HD200B
    PixelFormat48bppRGB = &H10300C
    PixelFormat64bppARGB = &H34400D
    PixelFormat64bppPARGB = &H1C400E
    PixelFormatGDI = &H20000
    PixelFormat1bppIndexed = &H30101
    PixelFormat4bppIndexed = &H30402
    PixelFormat8bppIndexed = &H30803
    PixelFormatAlpha = &H40000
    PixelFormatIndexed = &H10000
    PixelFormatPAlpha = &H80000
    PixelFormatExtended = &H100000
    PixelFormatCanonical = &H200000
End Enum

Public Enum WrapMode
    WrapModeTile = &H0
    WrapModeTileFlipX = &H1
    WrapModeTileFlipy = &H2
    WrapModeTileFlipXY = &H3
    WrapModeClamp = &H4

End Enum


' ----==== GDI+ API Declarationen ====----
Public Declare Function GdiplusShutdown Lib "GdiPlus.dll" ( _
                        ByVal token As Long) As Status

Public Declare Function GdiplusStartup Lib "GdiPlus.dll" ( _
                        ByRef token As Long, _
                        ByRef lpInput As GDIPlusStartupInput, _
                        ByRef lpOutput As GdiplusStartupOutput) As Status

Public Declare Function GdipGetImagePixelFormat Lib "gdiplus" ( _
                        ByVal Image As Long, _
                        ByRef PixelFormat As PixelFormats) As Status

Public Declare Function GdipDrawImageRect Lib "gdiplus" ( _
                        ByVal graphics As Long, _
                        ByVal Image As Long, _
                        ByVal X As Single, _
                        ByVal Y As Single, _
                        ByVal Width As Single, _
                        ByVal Height As Single) As Status

Public Declare Function GdipDrawImageRectRect Lib "gdiplus" ( _
                        ByVal mGraphics As Long, _
                        ByVal mImage As Long, _
                        ByVal mDstx As Single, _
                        ByVal mDsty As Single, _
                        ByVal mDstwidth As Single, _
                        ByVal mDstheight As Single, _
                        ByVal mSrcx As Single, _
                        ByVal mSrcy As Single, _
                        ByVal mSrcwidth As Single, _
                        ByVal mSrcheight As Single, _
                        ByVal mSrcUnit As Unit, _
                        ByVal mImageAttributes As Long, _
                        ByVal mcallback As Long, _
                        ByVal mcallbackData As Long) As Status

Public Declare Function GdipLoadImageFromFile Lib "gdiplus" ( _
                        ByVal mFilename As Long, _
                        ByRef mImage As Long) As Status

Public Declare Function GdipDisposeImage Lib "gdiplus" ( _
                        ByVal Image As Long) As Status

Public Declare Function GdipGraphicsClear Lib "gdiplus" ( _
                        ByVal mGraphics As Long, _
                        ByVal mColor As Long) As Status

Public Declare Function GdipCreateBitmapFromScan0 Lib "gdiplus" ( _
                        ByVal Width As Long, _
                        ByVal Height As Long, _
                        ByVal Stride As Long, _
                        ByVal Format As PixelFormats, _
                        ByRef Scan0 As Any, _
                        ByRef Bitmap As Long) As Status

Public Declare Function GdipGetImageDimension Lib "gdiplus" ( _
                        ByVal Image As Long, _
                        ByRef Width As Single, _
                        ByRef Height As Single) As Status

Public Declare Function GdipGetImageGraphicsContext Lib "gdiplus" ( _
                        ByVal Image As Long, _
                        ByRef graphics As Long) As Status

Public Declare Function GdipCreateFromHDC Lib "GdiPlus.dll" ( _
                        ByVal mhDC As Long, _
                        ByRef mGraphics As Long) As Status

Public Declare Function GdipDeleteGraphics Lib "GdiPlus.dll" ( _
                        ByVal mGraphics As Long) As Status

Public Declare Function GdipCreatePath Lib "GdiPlus.dll" ( _
                        ByVal mBrushMode As FillMode, _
                        ByRef mpath As Long) As Status

Public Declare Function GdipDeletePath Lib "GdiPlus.dll" ( _
                        ByVal mpath As Long) As Status

Public Declare Function GdipAddPathRectangle Lib "GdiPlus.dll" ( _
                        ByVal mpath As Long, _
                        ByVal mX As Single, _
                        ByVal mY As Single, _
                        ByVal mWidth As Single, _
                        ByVal mHeight As Single) As Status

Public Declare Function GdipFillPath Lib "GdiPlus.dll" ( _
                        ByVal mGraphics As Long, _
                        ByVal mBrush As Long, _
                        ByVal mpath As Long) As Status

Public Declare Function GdipCreateTexture Lib "GdiPlus.dll" ( _
                        ByVal mImage As Long, _
                        ByVal mWrapMode As WrapMode, _
                        ByRef mTexture As Long) As Status

Public Declare Function GdipGetPathWorldBounds Lib "GdiPlus.dll" ( _
                        ByVal mpath As Long, _
                        ByRef mBounds As RECTF, _
                        ByVal mMatrix As Long, _
                        ByVal mPen As Long) As Status
                        
Public Declare Function GdipDeleteBrush Lib "GdiPlus.dll" ( _
                        ByVal mBrush As Long) As Status

Public Declare Function GdipCreateMatrix Lib "GdiPlus.dll" ( _
                        ByRef mMatrix As Long) As Status

Public Declare Function GdipDeleteMatrix Lib "GdiPlus.dll" ( _
                        ByVal mMatrix As Long) As Status

Public Declare Function GdipRotateMatrix Lib "GdiPlus.dll" ( _
                        ByVal mMatrix As Long, _
                        ByVal mAngle As Single, _
                        ByVal mOrder As MatrixOrder) As Status

Public Declare Function GdipTransformPath Lib "GdiPlus.dll" ( _
                        ByVal mpath As Long, _
                        ByVal mMatrix As Long) As Status

Public Declare Function GdipTranslateWorldTransform Lib "GdiPlus.dll" ( _
                        ByVal graphics As Long, _
                        ByVal dx As Single, _
                        ByVal dy As Single, _
                        ByVal order As MatrixOrder) As Status

Public Declare Function GdipRotateWorldTransform Lib "GdiPlus.dll" ( _
                        ByVal graphics As Long, _
                        ByVal angle As Single, _
                        ByVal order As MatrixOrder) As Status

Public Declare Function GdipResetWorldTransform Lib "GdiPlus.dll" ( _
                        ByVal graphics As Long) As Status

' ----==== Variablen ====----
Private GdipToken As Long

Public Function StartUpGDIPlus() As Status

    Dim GdipStartupInput As GDIPlusStartupInput
    Dim GdipStartupOutput As GdiplusStartupOutput

    GdipStartupInput.GdiPlusVersion = GdiPlusVersion

    StartUpGDIPlus = GdiplusStartup(GdipToken, GdipStartupInput, GdipStartupOutput)

End Function

Public Function ShutdownGDIPlus() As Status

    ShutdownGDIPlus = GdiplusShutdown(GdipToken)

End Function

Public Function LoadImagePlus(ByVal strFileName As String, _
                              ByRef lngOutBitmap As Long, _
                              ByRef sngOutWidth As Single, _
                              ByRef sngOutHeight As Single, _
                              ByRef eOutPixelFormat As PixelFormats) As Boolean

    Dim bolRet As Boolean
    Dim pstrFileName As Long
    Dim lngOrgBitmap As Long
    Dim lngGraphics As Long

    bolRet = False
    pstrFileName = StrPtr(strFileName)

    If Execute(GdipLoadImageFromFile(pstrFileName, lngOrgBitmap)) = OK Then
        
        If Execute(GdipGetImageDimension(lngOrgBitmap, sngOutWidth, sngOutHeight)) = OK _
            Then
            
            If Execute(GdipGetImagePixelFormat(lngOrgBitmap, eOutPixelFormat)) = OK Then
            
                ' Hier noch Bedingungen wenn PixelFormat zb ein Indexed Format ist.
            
                If Execute(GdipCreateBitmapFromScan0(CLng(sngOutWidth), CLng(sngOutHeight), _
                    0&, eOutPixelFormat, ByVal 0&, lngOutBitmap)) = OK Then
        
                    If Execute(GdipGetImageGraphicsContext(lngOutBitmap, lngGraphics)) = OK Then
                    
                        If Execute(GdipDrawImageRect(lngGraphics, lngOrgBitmap, 0, 0, _
                            sngOutWidth, sngOutHeight)) = OK Then
                    
                            bolRet = True
                    
                        End If
                
                        Call Execute(GdipDeleteGraphics(lngGraphics))
                    
                    End If
                
                End If
                
            End If
        
        End If
                        
        Call Execute(GdipDisposeImage(lngOrgBitmap))
        
    End If
    
    LoadImagePlus = bolRet
    
End Function

Public Function CreatePreviewBitmap(ByVal lngInBitmap As Long, _
                                    ByVal sngInWidth As Single, _
                                    ByVal sngInHeight As Single, _
                                    ByVal eInPixelFormat As PixelFormats, _
                                    ByRef sngOutWidth As Single, _
                                    ByRef sngOutHeight As Single, _
                                    ByRef lngOutBitmap As Long) As Boolean

    Dim bolRet As Boolean
    Dim sngWidth As Single
    Dim sngHeight As Single
    Dim sngScaleX As Single
    Dim sngScaleY As Single
    Dim sngScaleR As Single
    Dim sngDiagSize As Single
    Dim lngGraphics As Long

    bolRet = False
    
    If lngInBitmap <> 0& Then
    
        If Execute(GdipGetImageDimension(lngInBitmap, sngWidth, sngHeight)) = OK Then
            
            sngDiagSize = Sqr((sngWidth ^ 2) + (sngHeight ^ 2))
                    
            sngScaleX = sngInWidth / sngDiagSize
            sngScaleY = sngInHeight / sngDiagSize
                    
            If sngScaleX > sngScaleY Then sngScaleR = sngScaleY Else sngScaleR = sngScaleX
                    
            sngOutWidth = sngWidth * sngScaleR
            sngOutHeight = sngHeight * sngScaleR
            
            If Execute(GdipCreateBitmapFromScan0(CLng(sngOutWidth), CLng(sngOutHeight), 0&, _
                                                 eInPixelFormat, ByVal 0&, lngOutBitmap)) = OK Then
            
                If Execute(GdipGetImageGraphicsContext(lngOutBitmap, lngGraphics)) = OK Then
                    
                    If Execute(GdipDrawImageRectRect(lngGraphics, lngInBitmap, _
                                                     0, 0, sngOutWidth, sngOutHeight, _
                                                     0, 0, sngWidth, sngHeight, _
                                                     UnitPixel, 0&, 0&, 0&)) = OK Then
                    
                        bolRet = True
                    
                    End If
                
                Call Execute(GdipDeleteGraphics(lngGraphics))
            
                End If
            
            End If
    
        End If
    
    End If
    
    CreatePreviewBitmap = bolRet

End Function

Public Function CalcRotationBitmapSize(ByVal lngInBitmap As Long, _
                                       ByVal sngInAngle As Single, _
                                       ByVal sngInWidth As Single, _
                                       ByVal sngInHeight As Single, _
                                       ByRef sngOutWidth As Single, _
                                       ByRef sngOutHeight As Single) As Boolean

    Dim bolRet As Boolean
    Dim lngPath As Long
    Dim lngMatrix As Long
    Dim lngGraphics As Long
    Dim tRectF As RECTF
    
    bolRet = False
    
    If lngInBitmap <> 0& Then
    
        If Execute(GdipGetImageGraphicsContext(lngInBitmap, lngGraphics)) = OK Then
    
            If Execute(GdipCreatePath(FillModeAlternate, lngPath)) = OK Then
            
                If Execute(GdipAddPathRectangle(lngPath, 0, 0, sngInWidth, sngInHeight)) = OK Then
            
                    If Execute(GdipCreateMatrix(lngMatrix)) = OK Then
                        
                        If Execute(GdipRotateMatrix(lngMatrix, sngInAngle, MatrixOrderAppend)) = OK Then
                        
                            If Execute(GdipTransformPath(lngPath, lngMatrix)) = OK Then
                        
                                If Execute(GdipGetPathWorldBounds(lngPath, tRectF, 0&, 0&)) = OK Then
                                
                                    sngOutWidth = tRectF.Width
                                    sngOutHeight = tRectF.Height
                            
                                    bolRet = True
                            
                                End If
                        
                            End If
                        
                        End If
                        
                        Call Execute(GdipDeleteMatrix(lngMatrix))
                
                    End If
            
                End If
        
                Call Execute(GdipDeletePath(lngPath))
        
            End If
        
            Call Execute(GdipDeleteGraphics(lngGraphics))
        
        End If
    
    End If

    CalcRotationBitmapSize = bolRet
    
End Function

Public Function ShowPreviewRotationBitmap(ByVal lngInBitmap As Long, _
                                          ByVal sngInAngle As Single, _
                                          ByVal sngInWidth As Single, _
                                          ByVal sngInHeight As Single, _
                                          ByVal eInPixelFormat As PixelFormats, _
                                          ByVal oPB As PictureBox) As Boolean

    Dim bolRet As Boolean
    Dim sngWidth As Single
    Dim sngHeight As Single
    Dim lngRotationBitmap As Long
    Dim lngGraphics As Long

    bolRet = False

    If lngInBitmap <> 0& Then

        If Execute(GdipGetImageDimension(lngInBitmap, sngWidth, sngHeight)) = OK _
            Then

            If Execute(GdipCreateBitmapFromScan0(sngInWidth, sngInHeight, 0&, _
                eInPixelFormat, ByVal 0&, lngRotationBitmap)) = OK Then

                If Execute(GdipGetImageGraphicsContext(lngRotationBitmap, _
                    lngGraphics)) = OK Then

                    If Execute(GdipTranslateWorldTransform(lngGraphics, sngInWidth _
                        / 2, sngInHeight / 2, MatrixOrderAppend)) = OK Then

                        If Execute(GdipRotateWorldTransform(lngGraphics, 180 + _
                            sngInAngle, MatrixOrderPrepend)) = OK Then

                            Call Execute(GdipDrawImageRect(lngGraphics, _
                                lngInBitmap, sngWidth / 2, sngHeight / 2, _
                                -sngWidth, -sngHeight))

                        End If

                    End If

                    Call Execute(GdipDeleteGraphics(lngGraphics))

                End If

                If Execute(GdipCreateFromHDC(oPB.hDC, lngGraphics)) = OK Then
                
                    If Execute(GdipGraphicsClear(lngGraphics, oPB.BackColor Or &HFF000000)) = OK Then

                        Call Execute(GdipDrawImageRect(lngGraphics, lngRotationBitmap, _
                            (oPB.ScaleWidth - sngInWidth) / 2, ( _
                            oPB.ScaleHeight - sngInHeight) / 2, sngInWidth, _
                            sngInHeight))

                        Call Execute(GdipDeleteGraphics(lngGraphics))
                        
                        oPB.Picture = oPB.Image
                        
                    End If

                End If
                
                Call Execute(GdipDisposeImage(lngRotationBitmap))

            End If

        End If

    End If

    ShowPreviewRotationBitmap = bolRet

End Function


Public Function Execute(ByVal eReturn As Status) As Status

    Dim eCurErr As Status

    If eReturn = OK Then

        eCurErr = OK

    Else

        eCurErr = eReturn

        MsgBox GdiErrorString(eReturn) & " GDI+ Error:" & eReturn, vbOKOnly, _
            "GDI+ Error"

    End If

    Execute = eCurErr

End Function

Private Function GdiErrorString(ByVal eError As Status) As String

    Dim s As String

    Select Case eError

    Case GenericError:              s = "Generic Error."
    Case InvalidParameter:          s = "Invalid Parameter."
    Case OutOfMemory:               s = "Out Of Memory."
    Case ObjectBusy:                s = "Object Busy."
    Case InsufficientBuffer:        s = "Insufficient Buffer."
    Case NotImplemented:            s = "Not Implemented."
    Case Win32Error:                s = "Win32 Error."
    Case WrongState:                s = "Wrong State."
    Case Aborted:                   s = "Aborted."
    Case FileNotFound:              s = "File Not Found."
    Case ValueOverflow:             s = "Value Overflow."
    Case AccessDenied:              s = "Access Denied."
    Case UnknownImageFormat:        s = "Unknown Image Format."
    Case FontFamilyNotFound:        s = "FontFamily Not Found."
    Case FontStyleNotFound:         s = "FontStyle Not Found."
    Case NotTrueTypeFont:           s = "Not TrueType Font."
    Case UnsupportedGdiplusVersion: s = "Unsupported Gdiplus Version."
    Case GdiplusNotInitialized:     s = "Gdiplus Not Initialized."
    Case PropertyNotFound:          s = "Property Not Found."
    Case PropertyNotSupported:      s = "Property Not Supported."
    Case ProfileNotFound:           s = "Profile Not Found."
    Case Else:                      s = "Unknown GDI+ Error."

    End Select

    GdiErrorString = s

End Function


